#
#  Representation of a Variable
#

package Variable;
use Util qw(util_split);
use strict;

# Variable->new(expand, append, value....)
#   Create new variable.
#   expand=1: expand this in output, variable does not appear
#   expand=0: retain as variable in output
#   append=1: the first assignment is "+=" style
#   append=0: the first assignment is "=" style
#   value: list of Value objects
sub new {
    my ($proto, $expand, $append, @value) = @_;
    bless({ expand => $expand,
            append => $append,
            value => [@value] },
          ref($proto) || $proto)
}

# Append a value
sub addValue {
    my $self = shift;
    push @{$self->{value}}, @_;
}

# Get value as string
sub getStringValue {
    my $self = shift;
    join (' ', map{$_->getValue()} @{$self->{value}});
}

# Get value as list
sub getListValue {
    my $self = shift;
    map {util_split($_->getValue())} @{$self->{value}};
}

sub isExpand {
    shift->{expand}
}

sub isAppend {
    shift->{append}
}

# Clone
sub clone {
    my $self = shift;
    Variable->new($self->{expand},
                  $self->{append},
                  map{$_->clone()} @{$self->{value}});
}

# Merge this with one or more other variables. Assumes that the other
# variable need not remain exclusive and unchanged, i.e. it's part of
# a "scratch" object. Returns the new object, which might be different
# from this one.
sub merge {
    my $self = shift;
    foreach (@_) {
        if ($_->isAppend()) {
            push @{$self->{value}}, @{$_->{value}};
        } else {
            $self = $_;
        }
    }
    $self;
}

1;
